<?php

namespace Drupal\book_moderation_sync\Hook;

use Drupal\book\BookManagerInterface;
use Drupal\book\BookOutlineStorageInterface;
use Drupal\content_moderation\ModerationInformationInterface;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Hook\Attribute\Hook;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\node\NodeInterface;

/**
 * Hooks for book moderation.
 */
class BookModerationHooks {

  /**
   * The logger channel.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * Constructs a new BookModerationHooks object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param \Drupal\content_moderation\ModerationInformationInterface $moderationInformation
   *   The moderation information service.
   * @param \Drupal\book\BookOutlineStorageInterface $bookOutlineStorage
   *   The book outline storage.
   * @param \Drupal\book\BookManagerInterface $bookManager
   *   The book manager service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   */
  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ModerationInformationInterface $moderationInformation,
    protected BookOutlineStorageInterface $bookOutlineStorage,
    protected BookManagerInterface $bookManager,
    LoggerChannelFactoryInterface $logger_factory,
  ) {
    $this->logger = $logger_factory->get('book_moderation_sync');
  }

  /**
   * Implements hook_node_presave().
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The node being saved.
   */
  #[Hook('node_presave')]
  public function nodePresave(NodeInterface $entity) {
    if ($entity->getType() === 'book' && $this->moderationInformation->isModeratedEntity($entity) && $entity->id()) {
      $this->syncModerationState($entity, $entity->get('moderation_state')->value);
    }
  }

  /**
   * Synchronizes the moderation state of child nodes.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The parent node.
   * @param string $state
   *   The moderation state to sync.
   */
  protected function syncModerationState(NodeInterface $node, $state) {
    foreach ($this->loadBookChildren($node) as $child_entity) {
      if ($this->moderationInformation->isModeratedEntity($child_entity) && $child_entity->get('moderation_state')->value !== $state) {
        $child_entity->set('moderation_state', $state);
        try {
          $child_entity->save();
        }
        catch (EntityStorageException $e) {
          $this->logError('Failed to sync moderation state for child node @nid: @message', [
            '@nid' => $child_entity->id(),
            '@message' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Deletes child nodes of a book.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The parent node.
   */
  public function deleteBookChildren(NodeInterface $node) {
    foreach ($this->loadBookChildren($node) as $child_entity) {
      try {
        $child_entity->delete();
      }
      catch (EntityStorageException $e) {
        $this->logError('Failed to delete child node @nid: @message', [
          '@nid' => $child_entity->id(),
          '@message' => $e->getMessage(),
        ]);
      }
    }
  }

  /**
   * Loads child nodes of a book.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The parent node.
   *
   * @return \Generator
   *   A generator yielding child nodes.
   */
  private function loadBookChildren(NodeInterface $node): \Generator {
    if (!$node->id()) {
      $this->logError('Cannot load children for a node without an ID.');
    }

    $children = $this->bookOutlineStorage->loadBookChildren($node->id());
    foreach ($children as $child_id => $data) {
      if (!isset($data['depth'])) {
        $this->logWarning('Node @nid has no "depth" key in its book structure.', ['@nid' => $child_id]);
        continue;
      }

      $child_entity = $this->entityTypeManager->getStorage('node')->load($child_id);
      if ($child_entity) {
        yield $child_entity;
      }
    }
  }

  /**
   * Logs an error message.
   *
   * @param string $message
   *   The message to log.
   * @param array $context
   *   The context for the message.
   */
  private function logError(string $message, array $context = []) {
    $this->logger->error($message, $context);
  }

  /**
   * Logs a warning message.
   *
   * @param string $message
   *   The message to log.
   * @param array $context
   *   The context for the message.
   */
  private function logWarning(string $message, array $context = []) {
    $this->logger->warning($message, $context);
  }

}
