<?php

namespace Drupal\Tests\book_moderation_sync\Functional;

use Drupal\user\Entity\User;
use Drupal\node\Entity\Node;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\content_moderation\Traits\ContentModerationTestTrait;

/**
 * Tests the Book Moderation Sync functionality.
 *
 * @group book_moderation_sync
 */
class BookModerationSyncTest extends BrowserTestBase {

  use ContentModerationTestTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'node',
    'book',
    'content_moderation',
    'book_moderation_sync',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * The book manager service.
   *
   * @var \Drupal\book\BookManagerInterface
   */
  protected $bookManager;

  /**
   * The book outline storage service.
   *
   * @var \Drupal\book\BookOutlineStorageInterface
   */
  protected $bookOutlineStorage;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->drupalCreateContentType(['type' => 'chapter', 'name' => 'Chapter']);

    $config = $this->config('book.settings');
    $allowed_types = $config->get('allowed_types');
    if (!in_array('chapter', $allowed_types)) {
      $allowed_types[] = 'chapter';
      $config->set('allowed_types', $allowed_types)->save();
    }

    $this->config('book_moderation_sync.settings')
      ->set('delete_children_on_book_delete', TRUE)
      ->save();

    $workflow = $this->createEditorialWorkflow();
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'book');
    $workflow->getTypePlugin()->addEntityTypeAndBundle('node', 'chapter');
    $workflow->save();

    $this->bookManager = $this->container->get('book.manager');
    $this->bookOutlineStorage = $this->container->get('book.outline_storage');

    $this->adminUser = User::load(1);
    $this->drupalLogin($this->adminUser);
  }

  /**
   * Tests that moderation states are synchronized between books and chapters.
   */
  public function testModerationStateSync() {
    $book = $this->drupalCreateNode([
      'type' => 'book',
      'title' => 'Test Book',
      'moderation_state' => 'draft',
    ]);

    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
      'moderation_state' => 'draft',
    ]);

    $chapter->book = [
      'bid' => $book->id(),
      'pid' => $book->id(),
      'weight' => 0,
      'has_children' => 0,
      'depth' => 1,
      'p1' => $book->id(),
      'p2' => 0,
      'p3' => 0,
      'p4' => 0,
      'p5' => 0,
      'p6' => 0,
      'p7' => 0,
      'p8' => 0,
      'p9' => 0,
    ];
    $chapter->save();
    $this->bookManager->updateOutline($chapter);

    $book_children = $this->bookOutlineStorage->loadBookChildren($book->id());
    $this->assertArrayHasKey($chapter->id(), $book_children, 'Chapter is correctly associated with the book.');

    $book->set('moderation_state', 'published');
    $book->save();

    $chapter = Node::load($chapter->id());
    $this->assertEquals('published', $chapter->get('moderation_state')->value, 'Chapter moderation state is synchronized with the book.');
  }

  /**
   * Tests deletion of child nodes when a book is deleted.
   */
  public function testDeleteBookChildren() {
    $book = $this->drupalCreateNode([
      'type' => 'book',
      'title' => 'Test Book',
    ]);

    $chapter = $this->drupalCreateNode([
      'type' => 'chapter',
      'title' => 'Test Chapter',
    ]);

    $chapter->book = [
      'bid' => $book->id(),
      'pid' => $book->id(),
    ];
    $chapter->save();
    $this->bookManager->updateOutline($chapter);

    $this->assertNotNull(Node::load($chapter->id()), 'Child node exists before book deletion.');

    $book->delete();

    $this->assertNull(Node::load($chapter->id()), 'Child node is deleted after book deletion.');
  }

}
